/* Erstellt 2022-06-07
 * Version V1.2
 * Autor:  Roland Kaiser, OE2ROL
 * Geschrieben für Arduino Nano ATmega 328P(Old Bootloader)  
 * Der Autor übernimmt keine Haftung etwaiger Folgeschäden an Transceivern oder dgl.
 * 
 * Diese Software ist für den eigenen privaten Gebrauch frei verfügbar, jedoch nicht für kommerzielle Zwecke ohne Genehmigung des Autors.
 * Veränderungen und Verbesserungen der Software bitte wieder an den Autor Roland Kaiser OE2ROL (roland.kaiser@sbg.at) senden.
 * 
 * Funktionsweise:
 * Manuelle Steuerung für Symmetrischen Koppler
 * Die C Werte (11 Bit) und L Werte (11 Bit) und die C-Position (TRX,ANT) werden über Up und Down Taste mit wählbarer Schrittweite ausgewählt und am 20x4 Display angezeigt
 * und über OneWire Bus an 3 Stk 8-Channel DS-2408 zum Symmetrischen Antennenkoppler gesendet.
 * 
 * Änderungsübersicht:
 * V1.0 initiale Version
 * V1.1 Ausgabe der Version in der Initmeldung
 * V1.2 Anpassung der LCD Ports an IC9700 Transverterboard
 * 
 * Anschlussbelegung der Ports:
 * One Wire Bus an D11
 * 
 * Display:     Arduino:
 * 4(RS)    ->  7
 * 6(E)     ->  10
 * 11(D4)   ->  5
 * 12(D5)   ->  4
 * 13(D6)   ->  6
 * 14(D7)   ->  2
 * LCD R/W pin 5 an ground
 * LCD VSS pin 1 an ground
 * LCD VCC pin 2 an 5V
 * 
 * 10K Poti (Kontrast):
 * Enden an +5V and ground
 * Schleifer an LCD VO pin 3
 * 
 * Tastatur:
 * Tastenport Step    -> Arduino Pin A2
 * Tastenport up      -> Arduino Pin A3
 * Tastenport down    -> Arduino Pin A4
 * Tastenport Menü    -> Arduino Pin A5
 * Die Tasten werden gegen Masse angeschlossen.
 */

#include <OneWire.h>
#include <LiquidCrystal.h>

const int rs = 7, en = 10, d4 = 5, d5 = 4, d6 = 6, d7 = 2;  // LCD Pinbelegung
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);                  // LCD Pinzuweisung
 
OneWire net(11);                                            // pin D11
byte IC1[8] =  {0x29,0x7A,0x8F,0x1B,0x00,0x00,0x00,0x7C};   // Adresse IC1 8x8 byte Array
byte IC2[8] =  {0x29,0x8A,0x8E,0x1B,0x00,0x00,0x00,0xC3};   // Adresse IC2 8x8 byte Array
byte IC3[8] =  {0x29,0x2A,0x8E,0x1B,0x00,0x00,0x00,0x9F};   // Adresse IC3 8x8 byte Array

byte a = 0;
byte b;
byte c = 0;
byte d;
byte e = 0;
byte f;
uint32_t C = 0;
uint32_t L = 0;
int io = 1;
float cap = 0;
uint32_t ind = 0;
uint32_t Cmem = 0;
uint32_t Lmem = 0;
uint32_t iomem = 0;
const int menuPin = A5;               // Pin für Menue Taste
const int downpin = A4;               // Pin für Up Taste
const int uppin = A3;                 // Pin für Down Taste
const int steppin = A2;               // Pin für STEP Taste
int menuPushCounter = 0;              // Menütastenzähler
int menuState = 0;                    // Menüstatus
int lastmenuState = 0;                // letzter Menü Status
boolean menubool = false;             // Menü initial deaktivieren
int upState;                          // up Status 
int up;                               // up Taste
int down;                             // Taste down
int STEP;                             // Taste STEP
int menucounter = 0;                  // Menü Zähler
int sw = 1;                           // Schrittweite im Menü

//********************************************* Setup ************************************************

void setup() {
  
  Serial.begin(9600);                   // Ausgang Serial
  lcd.begin(20, 4);                     // Ausgang LCD.
  pinMode(menuPin, INPUT);              // Init für die Tasten Pins
  digitalWrite(menuPin, HIGH);          // Init für die Tasten Pins
  pinMode(downpin, INPUT);              // Init für die Tasten Pins
  digitalWrite(downpin, HIGH);          // Init für die Tasten Pins
  pinMode(uppin, INPUT);                // Init für die Tasten Pins
  digitalWrite(uppin, HIGH);            // Init für die Tasten Pins
  pinMode(steppin, INPUT);              // Init für die Tasten Pins
  digitalWrite(steppin, HIGH);          // Init für die Tasten Pins

  a = C;
  c = (C >> 8)|(io << 3)| ((L >> 8) << 5);
  e = L;
  
  ic1();
  ic2();
  ic3();
  lcd.clear();                           // LCD löschen
  lcd.setCursor(0, 0);                   // setzt Cursor 1.Zeichen, 1.Zeile
  lcd.print("**** Symmetric ****");      // schreiben
  lcd.setCursor(0, 1);                   // setzt Cursor 1.Zeichen, 2.Zeile
  lcd.print("* Antenna Coupler *");      // schreiben
  lcd.setCursor(0, 2);                   // setzt Cursor 1.Zeichen, 3.Zeile
  lcd.print("***** OE2ROL ******");      // schreiben
  lcd.setCursor(0, 3);                   // setzt Cursor 1.Zeichen, 4.Zeile
  lcd.print("V1.2");                     // schreiben
  delay(4000);                           // ms warten
  lcd.clear();                           // LCD löschen
  
}

//****************************************** Loop **************************************************

void loop() {

  a = C;
  c = (C >> 8)|(io << 3)| ((L >> 8) << 5);
  e = L;

  menuset();                             // Menü ausführen
  
  
  lcd.setCursor(0, 0);                   // setzt Cursor 1.Zeichen, 1.Zeile
  lcd.print("Capacity:");                // schreiben
  lcd.setCursor(10, 0);                  // setzt Cursor 10.Zeichen, 1.Zeile
  lcd.print(cap);                        // schreibe Kapazitätswert
  lcd.setCursor(18, 0);                  // setzt Cursor 18.Zeichen, 1.Zeile
  lcd.print("pF");                       // schreiben
  lcd.setCursor(0, 1);                   // setzt Cursor 1.Zeichen, 2.Zeile
  lcd.print("Inductance:");              // schreiben
  lcd.setCursor(12, 1);                  // setzt Cursor 12.Zeichen, 2.Zeile
  lcd.print(ind);                        // schreibe Induktivitätswert
  lcd.setCursor(18, 1);                  // setzt Cursor 18.Zeichen, 2.Zeile
  lcd.print("nH");                       // schreiben
  lcd.setCursor(0, 2);                   // 1. Zeichen, 3. Zeile
  lcd.print("Cap. Pos.:");               // schreiben
      if (io == 1){
          lcd.setCursor(11, 2);          // 12. Zeichen, 3. Zeile
          lcd.print("ANT");              // schreiben
          }
      if (io == 2){
          lcd.setCursor(11, 2);          // 12. Zeichen, 3. Zeile
          lcd.print("TRX");              // schreiben
          }
  lcd.setCursor(0, 3);                   // 1. Zeichen, 4. Zeile
  lcd.print("MENU");                     // schreiben

Cmem = C;
Lmem = L;
iomem = io;

if (C != Cmem){
  ic1();
  ic2();
  lcd.clear();                           // LCD löschen
}

if (L != Lmem){
  ic2();
  ic3();
  lcd.clear();                           // LCD löschen
}

if (io != iomem){
  ic2();
  lcd.clear();                           // LCD löschen
}

}

// ************************************** Ansteuerung ÍC 1 **********************************************************

void ic1() {
  net.reset();                  // zurücksetzen aller 1-wire Geräte
  net.write(0x55);              // Slave auswählen zum Empfang der Adressbytes
  
        for(int i=0;i<8;i++)    // 8x8 byte Adressbytes aus dem Array lesen
        {
            net.write(IC1[i]);  // Adressbytes an den Slave senden
        }
    net.write(0x5A);            // Befehl zum Senden der Adressbytes (nur das angesprochene Gerät)
    delay(10);                  // ms warten
    b=~a;                       // Ausgabe invertieren
    net.write(a);               // zusammengeführte Ports an den 8Ch Switch senden
    net.write(b);               // invertierte Ausgabe an den 8Ch Switch senden
    net.write(0x0089);          // gespeicherte Schaltzustände am 8Ch Switch ausgeben
}

// ************************************** Ansteuerung ÍC 2 **********************************************************

void ic2() {
  net.reset();                  // zurücksetzen aller 1-wire Geräte
  net.write(0x55);              // Slave auswählen zum Empfang der Adressbytes
  
        for(int i=0;i<8;i++)    // 8x8 byte Adressbytes aus dem Array lesen
        {
            net.write(IC2[i]);  // Adressbytes an den Slave senden
        }
    net.write(0x5A);            // Befehl zum Senden der Adressbytes (nur das angesprochene Gerät)
    delay(10);                  // ms warten
    d=~c;                       // Ausgabe invertieren
    net.write(c);               // zusammengeführte Ports an den 8Ch Switch senden
    net.write(d);               // invertierte Ausgabe an den 8Ch Switch senden
    net.write(0x0089);          // gespeicherte Schaltzustände am 8Ch Switch ausgeben
}

// ************************************** Ansteuerung ÍC 3 **********************************************************

void ic3() {
  net.reset();                  // zurücksetzen aller 1-wire Geräte
  net.write(0x55);              // Slave auswählen zum Empfang der Adressbytes
  
        for(int i=0;i<8;i++)    // 8x8 byte Adressbytes aus dem Array lesen
        {
            net.write(IC3[i]);  // Adressbytes an den Slave senden
        }
    net.write(0x5A);            // Befehl zum Senden der Adressbytes (nur das angesprochene Gerät)
    delay(10);                  // ms warten
    f=~e;                       // Ausgabe invertieren
    net.write(e);               // zusammengeführte Ports an den 8Ch Switch senden
    net.write(f);               // invertierte Ausgabe an den 8Ch Switch senden
    net.write(0x0089);          // gespeicherte Schaltzustände am 8Ch Switch ausgeben
}  

// ************************************** Menüsteuerung **********************************************************

void menuset(){
menuState = digitalRead(menuPin);                       // Auslesen der Menütaste
  if (menuState != lastmenuState)                       // wenn sich der Menü Status geändert hat
  {
    if (menuState == LOW)                               // wenn Menü Status LOW (Menütaste gedrückt)
    {
      delay(200);                                       // Verzögerung
      menuPushCounter = 4;                              // maximal 4 Menüs verfügbar Menütastenzähler auf 4 setzen
      menubool = true;                                  // menü aktivieren
    }
    if (menuState == HIGH)                              // wenn Menü Status HIGH
    {
      menubool = false;                                 // menü deaktivieren
    }
  }

  if (menubool == true && menuPushCounter == 4)         // wenn Menü aktiviert und Menütastenzähler 4
  {
    menucounter = 0;                                    // Menüzähler auf 0 setzen 
    menu();                                             // Menü starten
  }
lastmenuState = menuState;                              // Menüstatus setzen
}



// *****************************Menüsteuerung zum Setzen der Werte****************************************

void menu()
{
  lcd.clear();                                    // LCD löschen
  delay(100);                                     // Verzögerung
  while (menucounter < 3)                         // while Menu enspricht der max Anzahl der Menüs
  {
    menuState = digitalRead(menuPin);             // Menü Taste abfragen
    up = digitalRead(uppin);                      // up Taste abfragen
    down = digitalRead(downpin);                  // down Taste abfragen
    STEP = digitalRead(steppin);                  // STEP Taste abfragen

    if (menuState != lastmenuState)               // wenn sich der Menü Status geändert hat
    {
      lcd.clear();                                // LCD löschen
      if (menuState == LOW)                       // wenn Menü Status LOW
      {
        delay(200);                               // Verzögerung
        menucounter++;                            // Menüzähler um 1 erhöhen
      }
    }
lastmenuState = menuState;                        // letzten Menüstatus auf Menü Status setzen

    switch (menucounter) {                        // Menüauswahl
      case 0:                                     // Menü 1 Kapazitätswert setzen
        Step();
        lcd.setCursor(0, 0);                      // setzt Cursor 1.Zeichen, 1.Zeile
        lcd.print("Capacity [pF]");               // schreibe Zeichen
        lcd.setCursor(0, 1);                      // setzt Cursor 1.Zeichen, 2.Zeile
        lcd.print(cap);                           // schreibe Kapazitätswert
        lcd.setCursor(0, 2);                      // 1. Zeichen, 3. Zeile
        lcd.print("Stepwidth:");                  // schreiben
        lcd.setCursor(16, 2);                     // 17. Zeichen, 3. Zeile
        lcd.print(sw);                            // Schrittweite schreiben
        lcd.setCursor(0, 3);                      // 1. Zeichen, 4. Zeile
        lcd.print("MENU");                        // schreiben
        lcd.setCursor(6, 3);                      // 7. Zeichen, 4. Zeile
        lcd.print("Down");                        // schreiben
        lcd.setCursor(12, 3);                     // 13. Zeichen, 4. Zeile
        lcd.print("UP");                          // schreiben
        lcd.setCursor(16, 3);                     // 17. Zeichen, 4. Zeile
        lcd.print("STEP");                        // schreiben
        if (up == LOW)                            // wenn up Taste betätigt
            {
              delay(100);                         // Verzögerung
              C = C+sw;
              if (C > 2047) {C = 0;}              // Begenzung des Encoderzählers auf 2047 (11 Bit)
              cap = C*3.125;
              a = C;
              c = (C >> 8)|(io << 3)| ((L >> 8) << 5);
              ic1();
              ic2();
              lcd.clear();                        // LCD löschen
            }
        if (down == LOW)                          // wenn down Taste betätigt
            {
              delay(100);                         // Verzögerung
              C = C-sw;
              if (C > 2047) {C = 2047;}           // Begenzung des Encoderzählers auf 2047 (11 Bit)
              cap = C*3.125;
              a = C;
              c = (C >> 8)|(io << 3)| ((L >> 8) << 5);
              ic1();
              ic2();
              lcd.clear();                        // LCD löschen
            }
      break;                                      // aussteigen, dieses Menü beginnt dann von vorne

      case 1:                                     // Menü 2 Induktivitätswert setzen
        Step();
        lcd.setCursor(0, 0);                      // 1. Zeichen, 1. Zeile
        lcd.print("Inductance [nH]");             // schreiben
        lcd.setCursor(0, 1);                      // 1. Zeichen, 2. Zeile
        lcd.print(ind);                           // schreibe Induktivitätswert
        lcd.setCursor(0, 2);                      // 1. Zeichen, 3. Zeile
        lcd.print("Stepwidth:");                  // schreiben
        lcd.setCursor(16, 2);                     // 17. Zeichen, 3. Zeile
        lcd.print(sw);                            // Schrittweite schreiben
        lcd.setCursor(0, 3);                      // 1. Zeichen, 4. Zeile
        lcd.print("MENU");                        // schreiben
        lcd.setCursor(6, 3);                      // 7. Zeichen, 4. Zeile
        lcd.print("Down");                        // schreiben
        lcd.setCursor(12, 3);                     // 13. Zeichen, 4. Zeile
        lcd.print("UP");                          // schreiben
        lcd.setCursor(16, 3);                     // 17. Zeichen, 4. Zeile
        lcd.print("STEP");                        // schreiben
        if (up == LOW)                            // wenn up Taste betätigt
            {
              delay(100);                         // Verzögerung
              L = L+sw;
              if (L > 2047) {L = 0;}              // Begenzung des Encoderzählers auf 2047 (11 Bit)
              ind = L*40;
              c = (C >> 8)|(io << 3)| ((L >> 8) << 5);
              e = L;
              ic2();
              ic3();
              lcd.clear();                        // LCD löschen
            }  
         if (down == LOW)                         // wenn down Taste betätigt
            {
              delay(100);                         // Verzögerung
              L = L-sw;
              if (L > 2047) {L = 2047;}           // Begenzung des Encoderzählers auf 2047 (11 Bit)
              ind = L*40;
              c = (C >> 8)|(io << 3)| ((L >> 8) << 5);
              e = L;
              ic2();
              ic3();
              lcd.clear();                        // LCD löschen
            }       
      break;                                      // aussteigen, dieses Menü beginnt dann von vorne
        
      case 2:                                     // Menü 2 C TRX/ANT/OFF
        lcd.setCursor(0, 0);                      // 1. Zeichen, 1. Zeile
        lcd.print("Capasitor Position:");         // schreiben
        lcd.setCursor(0, 3);                      // 1. Zeichen, 4. Zeile
        lcd.print("MENU");                        // schreiben
        lcd.setCursor(6, 3);                      // 7. Zeichen, 4. Zeile
        lcd.print("TRX");                         // schreiben
        lcd.setCursor(12, 3);                     // 13. Zeichen, 4. Zeile
        lcd.print("ANT");                         // schreiben
        lcd.setCursor(17, 3);                     // 18. Zeichen, 4. Zeile
        lcd.print("OFF");                         // schreiben

          if (io == 0){
            lcd.setCursor(0, 1);                  // 1. Zeichen, 2. Zeile
            lcd.print("OFF");                     // schreiben
            }      
          if (io == 1){
            lcd.setCursor(0, 1);                  // 1. Zeichen, 2. Zeile
            lcd.print("ANT");                     // schreiben
            }
          if (io == 2){
            lcd.setCursor(0, 1);                  // 1. Zeichen, 2. Zeile
            lcd.print("TRX");                     // schreiben
            }
            
            if (up == LOW)                        // wenn up Taste betätigt
            {
              delay(200);                         // Verzögerung
              io = 1;                             // C = ANT Position
              c = (C >> 8)|(io << 3)| ((L >> 8) << 5);
              ic2();
              lcd.clear();                        // LCD löschen
            }
            if (down == LOW)                      // wenn down Taste betätigt
            {
              delay(200);                         // Verzögerung
              io = 2;                             // C = TRX Position
              c = (C >> 8)|(io << 3)| ((L >> 8) << 5);
              ic2();
              lcd.clear();                        // LCD löschen
            }
            if (STEP == LOW)                      // wenn STEP Taste betätigt
            {
              delay(200);                         // Verzögerung
              io = 0;                             // C = OFF Position
              c = (C >> 8)|(io << 3)| ((L >> 8) << 5);
              ic2();
              lcd.clear();                        // LCD löschen
            }
        break;                                    // aussteigen, dieses Menü beginnt dann von vorne
        
    }
  }
}

//*********************************** Schrittweite *******************************************************

void Step()
{

            if (STEP == LOW)                    // wenn STEP Taste betätigt
            {
              delay(200);                       // Verzögerung
              sw=sw*10;                         // Schrittweite um Faktor 10 erhöhen
              lcd.clear();                      // LCD löschen
              if (sw > 100) {sw=1;}             // min Begrenzung auf 1
              if (sw > 100) {sw=100;}           // max Begrenzung auf 100
            }
}
